@@ -1,5 +1,5 @@
 /*
- *  $Id: Config.h,v 1.3 2002/03/07 21:15:01 rousseau Exp $
+ *  $Id: Config.h,v 1.10 2004/02/27 17:57:45 rousseau Exp $
  *  ifd-GemPC410
  *
  *  Created by JL Giraud <jl.giraud@free.fr> on Sun Nov 19 2000.
@@ -11,7 +11,7 @@
 #ifndef __CONFIG_H__
 #define __CONFIG_H__
 
-// Define debug levels here
+/* Define debug levels here */
 
 /* Critical informations. Should always be active */
 #define DEBUG_LEVEL_CRITICAL
@@ -20,25 +20,24 @@
 #define DEBUG_LEVEL_INFO
 
 /* periodic info: messages caused by pcsclite scanning every second */
-#define DEBUG_LEVEL_PERIODIC
-
-/* log APDU and results */
-#define DEBUG_LEVEL_APDU
+//#define DEBUG_LEVEL_PERIODIC
 
 /* low level communications */
-#define DEBUG_LEVEL_COMM
+//#define DEBUG_LEVEL_COMM
 
-// debug messages goes to stderr instead of syslog
-#define DEBUG_STDERR
+/* Perform automatic PPS negociation
+ * unsed by default for safety instead of speed */
+//#define AUTOMATIC_PPS
 
-// GemPC410 specific
+/* GemPC410 specific */
 
 #define OpenPort OpenGemPC410
+#define OpenPortByName OpenGemPC410ByName
 #define ClosePort CloseGemPC410
 #define ReadPort ReadGBP
 #define WritePort WriteGBP
 
-#define READER_NAME "GemPC410"
+#define READER_NAME "GemPC41x"
 
 #endif
 
