@@ -1,6 +1,6 @@
 /*
  *  GemPC410Utils.c
- *  $Id: GemPC410Utils.c,v 1.6 2001/12/09 17:25:18 rousseau Exp $
+ *  $Id: GemPC410Utils.c,v 1.7 2004/01/20 13:41:17 rousseau Exp $
  *  GemPC410 dedicated functions
  *
  *  Created by giraud on Sat Oct 20 2001.
@@ -10,6 +10,8 @@
  *
  */
 
+#include <stdio.h>
+
 #include "pcscdefines.h"
 #include "ifdhandler.h"
 #include "GemCore.h"
@@ -20,12 +22,12 @@
 #include "GCdebug.h"
 #include "GCTransport.h"
 
-RESPONSECODE OpenGemPC410(DWORD lun, DWORD channel)
+RESPONSECODE OpenGemPC410ByName(DWORD lun, LPSTR dev_name)
 {
 	UCHAR os_version[IFD_LEN_VERSION + 2];
 	DWORD length;
 
-	if (OpenGBP(lun, channel) != STATUS_SUCCESS)
+	if (OpenGBP(lun, dev_name) != STATUS_SUCCESS)
 	{
 		DEBUG_CRITICAL("OpenGBP failed");
 		return IFD_COMMUNICATION_ERROR;
@@ -49,6 +51,36 @@ RESPONSECODE OpenGemPC410(DWORD lun, DWORD channel)
 	}
 
 	return IFD_SUCCESS;
+} /* OpenGemPC410ByName */
+
+RESPONSECODE OpenGemPC410(DWORD lun, DWORD channel)
+{
+	char dev_name[FILENAME_MAX];
+
+	/*
+	 * Conversion of old-style ifd-hanler 1.0 CHANNELID 
+	 */
+	if (channel == 0x0103F8)
+		channel = 1;
+	else
+		if (channel == 0x0102F8)
+			channel = 2;
+		else
+			if (channel == 0x0103E8)
+				channel = 3;
+			else
+				if (channel == 0x0102E8)
+					channel = 4;
+
+	if (channel < 0)
+	{
+		DEBUG_CRITICAL2("wrong port number: %d", (int) channel);
+		return STATUS_UNSUCCESSFUL;
+	}
+
+	sprintf(dev_name, "/dev/pcsc/%d", (int) channel);
+
+	return OpenGemPC410ByName(lun, dev_name);
 }	/* OpenGemPC410 */
 
 RESPONSECODE CloseGemPC410(DWORD lun)
