@@ -1,6 +1,6 @@
 /*
     main.c: main function used for IFDH debug
-    Copyright (C) 2001   Ludovic Rousseau
+    Copyright (C) 2001-2004   Ludovic Rousseau
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,41 +18,17 @@
 */
 
 /*
- * $Id: main.c,v 1.7 2002/03/03 18:33:21 rousseau Exp $
- *
- * $Log: main.c,v $
- * Revision 1.7  2002/03/03 18:33:21  rousseau
- * add hexa values of max length
- * add a new test (commented by default)
- *
- * Revision 1.6  2002/03/03 17:35:59  rousseau
- * changed default serial port
- *
- * Revision 1.5  2001/11/27 21:11:32  rousseau
- * Implement ISO 7816 Case 2, 3 and 4 with full length tests
- *
- * Revision 1.4  2001/11/27 19:49:28  rousseau
- * add a pcsc_error() function instead of code duplication
- * add ReaderTest (Java test applet) APDU for ISO 7816 Case 1
- *
- * Revision 1.3  2001/11/15 21:18:31  rousseau
- * add IFDHTransmitToICC test
- * print return values (SW)
- *
- * Revision 1.1  2001/10/25 21:14:45  rousseau
- * *** empty log message ***
- *
- * Revision 1.1.1.1  2001/10/22 21:36:15  rousseau
- * Imported sources
- *
- *
+ * $Id: main.c,v 1.16 2004/01/20 13:41:17 rousseau Exp $
  */
 
 #include <stdio.h>
 #include <unistd.h>
 #include <winscard.h>
+#include <string.h>
 
 #include "ifdhandler.h"
+#include "Config.h"
+#include "GCdebug.h"
 
 // CHANNELID:
 //  0x000001 - /dev/pcsc/1
@@ -62,6 +38,7 @@
 // 1 -> GCR400
 // 2 -> GemPC410
 #define SERIAL_PORT 2
+#define SERIAL_NAME "/dev/pcsc/2"
 
 #define LUN 0
 
@@ -76,12 +53,20 @@ int main(int argc, char *argv[])
 	DWORD dwSendLength, dwRecvLength;
 	SCARD_IO_HEADER SendPci, RecvPci;
 
+	rv = IFDHCreateChannelByName(LUN, SERIAL_NAME);
+	if (rv != IFD_SUCCESS)
+	{
+		printf("IFDHCreateChannel: %d\n", rv);
+		return 1;
+	}
+#if 0
 	rv = IFDHCreateChannel(LUN, SERIAL_PORT);
 	if (rv != IFD_SUCCESS)
 	{
 		printf("IFDHCreateChannel: %d\n", rv);
 		return 1;
 	}
+#endif
 
 	rv = IFDHICCPresence(LUN);
 	pcsc_error(rv);
@@ -90,42 +75,17 @@ int main(int argc, char *argv[])
 	if (rv != IFD_SUCCESS)
 	{
 		printf("IFDHPowerICC: %d\n", rv);
-		//return 1;
+
+		goto end;
 	}
 
-	printf("ATR (%ld bytes): ", atrlength);
-	for (i=0; i<atrlength; i++)
-		printf("%02X ", atr[i]);
-	printf("\n");
-
+	debug_xxd("ATR: ", atr, atrlength);
 
 	rv = IFDHICCPresence(LUN);
 	pcsc_error(rv);
 
-#if 0
-	/* Case 1 */
-	s[0] = 0x80;
-	s[1] = 0x23;
-	s[2] = 0x01;
-	s[3] = 0x00;
-	s[4] = 0xFF;
-
-	dwSendLength = 5;
-	dwRecvLength = sizeof(r);
-
-	rv = IFDHTransmitToICC(LUN, SendPci, s, dwSendLength, r, &dwRecvLength,
-		&RecvPci);
-
-	if (rv == 0)
-	{
-		printf("Received: ");
-		for (i=0; i<dwRecvLength; i++)
-			printf("%02X ", r[i]);
-		printf("\n");
-	}
-	else
-		pcsc_error(rv);
-#endif
+	memset(&SendPci, 0, sizeof(SendPci));
+	memset(&RecvPci, 0, sizeof(RecvPci));
 
 	/* select applet */
 	s[0] = 0x00;
@@ -143,20 +103,14 @@ int main(int argc, char *argv[])
 	dwSendLength = 11;
 	dwRecvLength = sizeof(r);
 
+	debug_xxd("Select applet: ", s, dwSendLength);
 	rv = IFDHTransmitToICC(LUN, SendPci, s, dwSendLength, r, &dwRecvLength,
 		&RecvPci);
 
-	if (rv == 0)
-	{
-		printf("Received: ");
-		for (i=0; i<dwRecvLength; i++)
-			printf("%02X ", r[i]);
-		printf("\n");
-	}
-	else
+	debug_xxd("Received: ", r, dwRecvLength);
+	if (rv)
 		pcsc_error(rv);
 
-
 	/* Case 1 */
 	s[0] = 0x80;
 	s[1] = 0x21;
@@ -167,17 +121,12 @@ int main(int argc, char *argv[])
 	dwSendLength = 5;
 	dwRecvLength = sizeof(r);
 
+	debug_xxd("Case 1: ", s, dwSendLength);
 	rv = IFDHTransmitToICC(LUN, SendPci, s, dwSendLength, r, &dwRecvLength,
 		&RecvPci);
 
-	if (rv == 0)
-	{
-		printf("Received: ");
-		for (i=0; i<dwRecvLength; i++)
-			printf("%02X ", r[i]);
-		printf("\n");
-	}
-	else
+	debug_xxd("Received: ", r, dwRecvLength);
+	if (rv)
 		pcsc_error(rv);
 
 	/* Case 2 */
@@ -199,17 +148,12 @@ int main(int argc, char *argv[])
 	dwSendLength = len_i + 5;
 	dwRecvLength = sizeof(r);
 
+	debug_xxd("Case 2: ", s, dwSendLength);
 	rv = IFDHTransmitToICC(LUN, SendPci, s, dwSendLength, r, &dwRecvLength,
 		&RecvPci);
 
-	if (rv == 0)
-	{
-		printf("Received: ");
-		for (i=0; i<dwRecvLength; i++)
-			printf("%02X ", r[i]);
-		printf("\n");
-	}
-	else
+	debug_xxd("Received: ", r, dwRecvLength);
+	if (rv)
 		pcsc_error(rv);
 
 	/* Case 3 */
@@ -236,17 +180,12 @@ int main(int argc, char *argv[])
 	dwSendLength = 5;
 	dwRecvLength = sizeof(r);
 
+	debug_xxd("Case 3: ", s, dwSendLength);
 	rv = IFDHTransmitToICC(LUN, SendPci, s, dwSendLength, r, &dwRecvLength,
 		&RecvPci);
 
-	if (rv == 0)
-	{
-		printf("Received: ");
-		for (i=0; i<dwRecvLength; i++)
-			printf("%02X ", r[i]);
-		printf("\n");
-	}
-	else
+	debug_xxd("Received: ", r, dwRecvLength);
+	if (rv)
 		pcsc_error(rv);
 
 	/* Case 4 */
@@ -282,17 +221,12 @@ int main(int argc, char *argv[])
 	dwSendLength = len_i + 5;
 	dwRecvLength = sizeof(r);
 
+	debug_xxd("Case 4: ", s, dwSendLength);
 	rv = IFDHTransmitToICC(LUN, SendPci, s, dwSendLength, r, &dwRecvLength,
 		&RecvPci);
 
-	if (rv == 0)
-	{
-		printf("Received: ");
-		for (i=0; i<dwRecvLength; i++)
-			printf("%02X ", r[i]);
-		printf("\n");
-	}
-	else
+	debug_xxd("Received: ", r, dwRecvLength);
+	if (rv)
 		pcsc_error(rv);
 
 	/* Get response */
@@ -305,19 +239,15 @@ int main(int argc, char *argv[])
 	dwSendLength = 5;
 	dwRecvLength = sizeof(r);
 
+	debug_xxd("Get response: ", s, dwSendLength);
 	rv = IFDHTransmitToICC(LUN, SendPci, s, dwSendLength, r, &dwRecvLength,
 		&RecvPci);
 
-	if (rv == 0)
-	{
-		printf("Received: ");
-		for (i=0; i<dwRecvLength; i++)
-			printf("%02X ", r[i]);
-		printf("\n");
-	}
-	else
+	debug_xxd("Received: ", r, dwRecvLength);
+	if (rv)
 		pcsc_error(rv);
 
+end:
 	/* Close */
 	rv = IFDHCloseChannel(LUN);
 	if (rv != IFD_SUCCESS)
@@ -334,19 +264,27 @@ void pcsc_error(int rv)
 	switch (rv)
 	{
 		case IFD_ICC_PRESENT:
-			printf("card present\n");
+			DEBUG_INFO("IFD: card present");
 			break;
 
 		case IFD_ICC_NOT_PRESENT:
-			printf("card _NOT_ present\n");
+			DEBUG_INFO("IFD: card _NOT_ present");
 			break;
 			
 		case IFD_COMMUNICATION_ERROR:
-			printf("communication error\n");
+			DEBUG_INFO("IFD: communication error");
+			break;
+
+		case IFD_PROTOCOL_NOT_SUPPORTED:
+			DEBUG_INFO("IFD: protocol not supported");
+			break;
+
+		case IFD_RESPONSE_TIMEOUT:
+			DEBUG_INFO("IFD: response timeout");
 			break;
 
 		default:
-			printf("undocumented error\n");
+			DEBUG_INFO2("IFD: undocumented error: %d", rv);
 	}
 } /* pcsc_error */
 
