@@ -1,5 +1,5 @@
 /*
- *  $Id: Config.h,v 1.6 2002/03/07 21:15:03 rousseau Exp $
+ *  $Id: Config.h,v 1.12 2004/02/27 17:57:47 rousseau Exp $
  *  ifd-GemPC430
  *
  *  Created by JL Giraud <jl.giraud@free.fr> on Sun Nov 19 2000.
@@ -10,7 +10,7 @@
 #ifndef __CONFIG_H__
 #define __CONFIG_H__
 
-// Define debug levels here
+/* Define debug levels here */
 
 /* Critical informations. Should always be active */
 #define DEBUG_LEVEL_CRITICAL
@@ -19,25 +19,24 @@
 #define DEBUG_LEVEL_INFO
 
 /* periodic info: messages caused by pcsclite scanning every second */
-#define DEBUG_LEVEL_PERIODIC
-
-/* log APDU and results */
-#define DEBUG_LEVEL_APDU
+//#define DEBUG_LEVEL_PERIODIC
 
 /* low level communications */
-#define DEBUG_LEVEL_COMM
+//#define DEBUG_LEVEL_COMM
 
-// debug messages goes to stderr instead of syslog
-#define DEBUG_STDERR
+/* Perform automatic PPS negociation
+ * unsed by default for safety instead of speed */
+//#define AUTOMATIC_PPS
 
-// GemPC430 specific
+/* GemPC430 specific */
  
 #define OpenPort OpenGemPC430
+#define OpenPortByName OpenGemPC430ByName
 #define ClosePort CloseGemPC430
 #define ReadPort ReadUSB
 #define WritePort WriteUSB
 
-#define READER_NAME "GemPC430"
+#define READER_NAME "GemPC43x"
 
 #endif
 
