@@ -1,31 +1,34 @@
 /*
  *  GemPC430Utils.c
- *  $Id: GemPC430Utils.c,v 1.14 2001/12/09 17:25:19 rousseau Exp $
+ *  $Id: GemPC430Utils.c,v 1.15 2004/01/20 13:42:15 rousseau Exp $
  *  GemPC430 dedicated functions
  *
  *  Created by giraud on Sat Oct 20 2001.
  *  Copyright (c) 2001 Jean-Luc Giraud.
+ *                2001-2004 Ludovic Rousseau
  *  License:   See file COPYING
  *
  */
 
+#include <stdio.h>
+
 #include "pcscdefines.h"
 #include "ifdhandler.h"
 #include "Config.h"
 #include "GemPC430Utils.h"
 #include "GemCore.h"
 #include "GCCmds.h"
-#include "usbserial.h"
+#include "libusb_wrap.h"
 #include "GCdebug.h"
 #include "GCTransport.h"
 
 
-ifd_t OpenGemPC430(DWORD lun, DWORD channel)
+ifd_t OpenGemPC430ByName(DWORD lun, LPSTR dev_name)
 {
 	UCHAR os_version[IFD_LEN_VERSION+2];
 	DWORD length;
 
-    if (OpenUSB(lun, channel) != STATUS_SUCCESS)
+    if (OpenUSB(lun, dev_name) != STATUS_SUCCESS)
     {
         DEBUG_CRITICAL("OpenUSB failed");
         return IFD_COMMUNICATION_ERROR;
@@ -51,6 +54,11 @@ ifd_t OpenGemPC430(DWORD lun, DWORD channel)
 	DEBUG_CRITICAL2("OS string: %s", os_version);
 
 	return IFD_SUCCESS;
+} /* OpenGemPC430ByName */
+
+ifd_t OpenGemPC430(DWORD lun, DWORD channel)
+{
+	return OpenGemPC430ByName(lun, NULL);
 } /* OpenGemPC430 */
 
 ifd_t CloseGemPC430(DWORD lun)
