@@ -3,8 +3,8 @@
 / File   :   ifdhandler.c
 / Authors :   David Corcoran <corcoran@linuxnet.com>
 /             Jean-Luc Giraud <jl.giraud@free.fr>
-/             Ludovic Rousseau <ludovic.rousseau@free.fr>, Oct-Nov 2001
-/ Date   :   April 9, 2001
+/             Ludovic Rousseau <ludovic.rousseau@free.fr>
+/ Date   :   April 9, 2001, 2002, 2003
 / Purpose:   This provides reader specific low-level calls
 /            for the GemPC family of Gemplus. The function
 /            stubs were written by D. Corcoran, the GemCore
@@ -13,12 +13,15 @@
 /            See http://www.linuxnet.com for more information.
 / License:   See file COPYING.BSD
 /
-/ $Id: ifdhandler.c,v 1.11 2002/03/07 21:35:46 rousseau Exp $
+/ $Id: ifdhandler.c,v 1.22 2004/01/20 13:58:59 rousseau Exp $
 /
 ******************************************************************/
 
 #include <stdio.h>
 #include <string.h>
+#ifdef HAVE_PTHREAD_H
+#include <pthread.h>
+#endif
 
 #include "pcscdefines.h"
 #include "ifdhandler.h"
@@ -41,7 +44,31 @@
 #endif
 
 // Array of structures to hold the ATR and other state value of each slot
-static GCoreDesc pgSlots[PCSCLITE_MAX_CHANNELS][MAX_SLOT_NB];
+static GCoreDesc pgSlots[PCSCLITE_MAX_READERS];
+
+
+RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPSTR lpcDevice)
+{
+	DEBUG_INFO3("lun: %X, device: %s", Lun, lpcDevice);
+
+	if (iLunCheck(Lun))
+		return IFD_COMMUNICATION_ERROR;
+
+	// Reset ATR buffer
+	pgSlots[LunToReaderIndex(Lun)].nATRLength = 0;
+	*pgSlots[LunToReaderIndex(Lun)].pcATRBuffer = '\0';
+
+	// Reset PowerFlags
+	pgSlots[LunToReaderIndex(Lun)].bPowerFlags = POWERFLAGS_RAZ;
+
+	if (OpenPortByName(Lun, lpcDevice) != IFD_SUCCESS)
+	{
+		DEBUG_CRITICAL("OpenPort failed");
+		return IFD_COMMUNICATION_ERROR;
+	}
+
+	return IFD_SUCCESS;
+} /* IFDHCreateChannelByName */
 
 
 RESPONSECODE IFDHCreateChannel(DWORD Lun, DWORD Channel)
@@ -58,8 +85,11 @@ RESPONSECODE IFDHCreateChannel(DWORD Lun, DWORD Channel)
 	 */
 
 	/*
-	 * Channel - Channel ID.  This is denoted by the following: 0x000001 -
-	 * /dev/pcsc/1 0x000002 - /dev/pcsc/2 0x000003 - /dev/pcsc/3
+	 * Channel - Channel ID.  This is denoted by the following:
+	 * 0x000001 - /dev/pcsc/1
+	 * 0x000002 - /dev/pcsc/2
+	 * 0x000003 - /dev/pcsc/3
+	 * 0x000004 - /dev/pcsc/4
 	 *
 	 * USB readers may choose to ignore this parameter and query the bus
 	 * for the particular reader.
@@ -79,27 +109,28 @@ RESPONSECODE IFDHCreateChannel(DWORD Lun, DWORD Channel)
 	 *
 	 * IFD_SUCCESS IFD_COMMUNICATION_ERROR
 	 */
+	RESPONSECODE return_value = IFD_SUCCESS;
 
-	DEBUG_INFO("Entering IFDHCreateChannel");
+	DEBUG_INFO3("lun: %X, channel: %d", Lun, Channel);
 
 	if (iLunCheck(Lun))
 		return IFD_COMMUNICATION_ERROR;
 
 	// Reset ATR buffer
-	pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].nATRLength = 0;
-	*pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].pcATRBuffer = '\0';
+	pgSlots[LunToReaderIndex(Lun)].nATRLength = 0;
+	*pgSlots[LunToReaderIndex(Lun)].pcATRBuffer = '\0';
 
 	// Reset PowerFlags
-	pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].bPowerFlags =
+	pgSlots[LunToReaderIndex(Lun)].bPowerFlags =
 		POWERFLAGS_RAZ;
 
 	if (OpenPort(Lun, Channel) != IFD_SUCCESS)
 	{
 		DEBUG_CRITICAL("OpenReader failed");
-		return IFD_COMMUNICATION_ERROR;
+		return_value = IFD_COMMUNICATION_ERROR;
 	}
 
-	return IFD_SUCCESS;
+	return return_value;
 } /* IFDHCreateChannel */
 
 
@@ -116,7 +147,7 @@ RESPONSECODE IFDHCloseChannel(DWORD Lun)
 	 * IFD_SUCCESS IFD_COMMUNICATION_ERROR
 	 */
 
-	DEBUG_INFO("entering IFDHCloseChannel");
+	DEBUG_INFO2("lun: %X", Lun);
 
 	if (iLunCheck(Lun))
 		return IFD_COMMUNICATION_ERROR;
@@ -125,6 +156,7 @@ RESPONSECODE IFDHCloseChannel(DWORD Lun)
 	// No reader status check, if it failed, what can you do ? :)
 
 	ClosePort(Lun);
+
 	return IFD_SUCCESS;
 } /* IFDHCloseChannel */
 
@@ -150,7 +182,7 @@ RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 	 * IFD_SUCCESS IFD_ERROR_TAG
 	 */
 
-	DEBUG_INFO("entering IFDHGetCapabilities\n");
+	DEBUG_INFO3("lun: %X, tag: %X", Lun, Tag);
 
 	if (iLunCheck(Lun))
 		return IFD_COMMUNICATION_ERROR;
@@ -161,14 +193,20 @@ RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 			// If Length is not zero, powerICC has been performed.
 			// Otherwise, return NULL pointer
 			// Buffer size is stored in *Length
-			*Length = (*Length < pgSlots[LunToReaderIndex(Lun)]
-				[GCoreLunToSlotNb(Lun)].nATRLength) ?
-				*Length : pgSlots[LunToReaderIndex(Lun)]
-				[GCoreLunToSlotNb(Lun)].nATRLength;
+			*Length = (*Length < pgSlots[LunToReaderIndex(Lun)].nATRLength) ?
+				*Length : pgSlots[LunToReaderIndex(Lun)].nATRLength;
 
 			if (*Length)
-				memcpy(Value, pgSlots[LunToReaderIndex(Lun)]
-					[GCoreLunToSlotNb(Lun)].pcATRBuffer, *Length);
+				memcpy(Value, pgSlots[LunToReaderIndex(Lun)].pcATRBuffer,
+					*Length);
+			break;
+
+		case TAG_IFD_SIMULTANEOUS_ACCESS:
+			if (*Length >= 1)
+			{
+				*Length = 1;
+				*Value = PCSCLITE_MAX_READERS;
+			}
 			break;
 
 		default:
@@ -200,7 +238,7 @@ RESPONSECODE IFDHSetCapabilities(DWORD Lun, DWORD Tag,
 
 	// By default, say it worked
 
-	DEBUG_PERIODIC("entering IFDHSetCapabilities");
+	DEBUG_PERIODIC3("lun: %X, tag: %X", Lun, Tag);
 
 	if (iLunCheck(Lun))
 		return IFD_COMMUNICATION_ERROR;
@@ -227,7 +265,7 @@ RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 	 * IFD_PROTOCOL_NOT_SUPPORTED
 	 */
 
-	DEBUG_INFO("entering IFDHSetProtocolParameters");
+	DEBUG_INFO2("lun: %X", Lun);
 
 	if (iLunCheck(Lun))
 		return IFD_COMMUNICATION_ERROR;
@@ -275,10 +313,11 @@ RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 	 * IFD_NOT_SUPPORTED
 	 */
 
-	DWORD nlength, rv;
+	DWORD nlength;
+	RESPONSECODE return_value = IFD_SUCCESS;
 	UCHAR pcbuffer[RESP_BUF_SIZE];
 
-	DEBUG_INFO("entering IFDHPowerICC");
+	DEBUG_INFO2("lun: %X", Lun);
 
 	// By default, assume it won't work :)
 	*AtrLength = 0;
@@ -291,46 +330,43 @@ RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 		case IFD_POWER_UP:
 		case IFD_RESET:
 			nlength = sizeof(pcbuffer);
-			if ((rv = GCCmdPowerUp(Lun, &nlength, pcbuffer)) != IFD_SUCCESS)
+			if ((return_value = GCCmdPowerUp(Lun, &nlength, pcbuffer)) != IFD_SUCCESS)
 			{
 				DEBUG_CRITICAL("PowerUp failed");
-				return rv;
+				goto end;
 			}
 
 			// Power up successful, set state variable to memorise it
-			pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-				bPowerFlags |= MASK_POWERFLAGS_PUP;
-			pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-				bPowerFlags &= ~MASK_POWERFLAGS_PDWN;
+			pgSlots[LunToReaderIndex(Lun)].bPowerFlags |= MASK_POWERFLAGS_PUP;
+			pgSlots[LunToReaderIndex(Lun)].bPowerFlags &= ~MASK_POWERFLAGS_PDWN;
 
 			// Reset is returned, even if TCK is wrong
-			pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-				nATRLength = *AtrLength =
+			pgSlots[LunToReaderIndex(Lun)].nATRLength = *AtrLength =
 				(nlength < MAX_ATR_SIZE) ? nlength : MAX_ATR_SIZE;
 			memcpy(Atr, pcbuffer, *AtrLength);
-			memcpy(pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-				pcATRBuffer, pcbuffer, *AtrLength);
+			memcpy(pgSlots[LunToReaderIndex(Lun)].pcATRBuffer, pcbuffer,
+				*AtrLength);
 
 			break;
 
 		case IFD_POWER_DOWN:
 			// Clear ATR buffer
-			pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].nATRLength =
+			pgSlots[LunToReaderIndex(Lun)].nATRLength =
 				0;
-			*pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-				pcATRBuffer = '\0';
+			*pgSlots[LunToReaderIndex(Lun)].pcATRBuffer = '\0';
 			// Memorise the request
-			pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].bPowerFlags
-				|= MASK_POWERFLAGS_PDWN;
+			pgSlots[LunToReaderIndex(Lun)].bPowerFlags |= MASK_POWERFLAGS_PDWN;
 			// send the command
-			return GCCmdPowerDown(Lun);
+			return_value = GCCmdPowerDown(Lun);
 			break;
 
 		default:
-			DEBUG_CRITICAL("IFDHPowerICC Action not supported");
-			return IFD_NOT_SUPPORTED;
+			DEBUG_CRITICAL("Action not supported");
+			return_value = IFD_NOT_SUPPORTED;
 	}
-	return IFD_SUCCESS;
+end:
+
+	return return_value;
 } /* IFDHPowerICC */
 
 
@@ -374,16 +410,21 @@ RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
 
 	DWORD ntestlength;
 	RESPONSECODE return_value = IFD_SUCCESS;	// Assume it will work
+	DWORD protocol;
+
+	DEBUG_INFO2("lun: %X", Lun);
 
-	DEBUG_INFO("entering IFDHTransmitToICC");
 	if (iLunCheck(Lun))
 		return IFD_COMMUNICATION_ERROR;
 
-#ifdef DEBUG_LEVEL_APDU
-	DEBUG_XXD("APDU sent: ", TxBuffer, TxLength);
-#endif
+	protocol = SendPci.Protocol;
 
-	switch (SendPci.Protocol)
+	// if the reader is in EMV mode use ISO_EXCHANGE instead of ISO_INPUT
+	// and ISO_OUTPUT. This is done with T=1 protocol
+	if (pgSlots[LunToReaderIndex(Lun)].bPowerFlags & MASK_POWERFLAGS_EMV)
+		protocol = T_1;
+
+	switch (protocol)
 	{
 		case T_0:
 			// Check if command is going to fit in buffer
@@ -416,7 +457,7 @@ RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
 					// set P3 to 0
 					cmd[ISO_CMD_SIZE] = 0;
 
-					return_value = gemcore_ISO_OUTPUT_processing(Lun,
+					return_value = gemcore_ISO_INPUT_processing(Lun,
 						cmd, ISO_CMD_SIZE + ISO_LENGTH_SIZE, RxBuffer,
 						RxLength);
 				}
@@ -483,11 +524,6 @@ clean_up_and_return:
 	if (return_value != IFD_SUCCESS)
 		*RxLength = 0;
 
-#ifdef DEBUG_LEVEL_APDU
-	if (*RxLength)
-		DEBUG_XXD("APDU result: ", RxBuffer, *RxLength);
-#endif
-
 	return return_value;
 
 } /* IFDHTransmitToICC */
@@ -511,7 +547,7 @@ RESPONSECODE IFDHControl(DWORD Lun, PUCHAR TxBuffer,
 	 * Notes: RxLength should be zero on error.
 	 */
 
-	DEBUG_INFO("entering IFDHControl");
+	DEBUG_INFO2("lun: %X", Lun);
 
 	if (iLunCheck(Lun))
 		return IFD_COMMUNICATION_ERROR;
@@ -531,9 +567,10 @@ RESPONSECODE IFDHICCPresence(DWORD Lun)
 	 */
 
 	UCHAR pcbuffer[GC_SIZE_CARD_STATUS];
+	RESPONSECODE return_value = IFD_COMMUNICATION_ERROR;
 	DWORD length;
 
-	DEBUG_PERIODIC("entering IFDHICCPresence");
+	DEBUG_PERIODIC2("lun: %X", Lun);
 
 	if (iLunCheck(Lun))
 		return IFD_COMMUNICATION_ERROR;
@@ -542,7 +579,8 @@ RESPONSECODE IFDHICCPresence(DWORD Lun)
 	if (GCCmdCardStatus(Lun, pcbuffer, &length) != IFD_SUCCESS)
 	{
 		DEBUG_CRITICAL("GCCmdCardStatus failed");
-		return IFD_COMMUNICATION_ERROR;
+		return_value = IFD_COMMUNICATION_ERROR;
+		goto clean_up_and_return;
 	}
 
 	if (GC_MASK_ICC_PRESENCE & pcbuffer[GC_OFFSET_STAT_BYTE])
@@ -554,26 +592,26 @@ RESPONSECODE IFDHICCPresence(DWORD Lun)
 			// Powered, so the ressource manager did not miss a quick
 			// removal/re-insertion
 			
-			return IFD_ICC_PRESENT;
+			return_value = IFD_ICC_PRESENT;
+			goto clean_up_and_return;
 		}
 		else
 		{
 			// Card present but not powered up
 			// Check if a power down has been requested
-			if (pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-				bPowerFlags & MASK_POWERFLAGS_PDWN)
+			if (pgSlots[LunToReaderIndex(Lun)].bPowerFlags & MASK_POWERFLAGS_PDWN)
 			{
 				DEBUG_PERIODIC("Card present not powered, power down requested");
 				// Powerdown requested, so situation is normal
 
-				return IFD_ICC_PRESENT;
+				return_value = IFD_ICC_PRESENT;
+				goto clean_up_and_return;
 			}
 
 			// Card inserted, not powered on but power down has not been
 			// requested
 			// Has the card been powered up already?
-			if (pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-				bPowerFlags & MASK_POWERFLAGS_PUP)
+			if (pgSlots[LunToReaderIndex(Lun)].bPowerFlags & MASK_POWERFLAGS_PUP)
 			{
 				DEBUG_PERIODIC("Card pull-out+re-insert detected CARD OUT SIMULATION");
 				// Power-up has been requested, but not power down and power is
@@ -581,44 +619,47 @@ RESPONSECODE IFDHICCPresence(DWORD Lun)
 				// out and reinserted too quickly for the resource manager to
 				// realise. A card out event is therefore simulated. Clear ATR
 				// buffer
-				pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-					nATRLength = 0;
-				*pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-					pcATRBuffer = '\0';
+				pgSlots[LunToReaderIndex(Lun)].nATRLength = 0;
+				*pgSlots[LunToReaderIndex(Lun)].pcATRBuffer = '\0';
 				// reset power flags
-				pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-					bPowerFlags = POWERFLAGS_RAZ;
+				pgSlots[LunToReaderIndex(Lun)].bPowerFlags = POWERFLAGS_RAZ;
 
-				return IFD_ICC_NOT_PRESENT;
+				return_value = IFD_ICC_NOT_PRESENT;
+				goto clean_up_and_return;
 			}
 
 			DEBUG_PERIODIC("Card present, just inserted");
 			// If control gets here, the card is in, not powered on, with
 			// no power down request and no previous power up request
 			// it is therefore a card insertion event
-			pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-				nATRLength = 0;
-			*pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-				pcATRBuffer = '\0';
+			pgSlots[LunToReaderIndex(Lun)].nATRLength = 0;
+			*pgSlots[LunToReaderIndex(Lun)].pcATRBuffer = '\0';
 			// reset power flags
-			pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-				bPowerFlags = POWERFLAGS_RAZ;
+			pgSlots[LunToReaderIndex(Lun)].bPowerFlags = POWERFLAGS_RAZ;
 			
-			return IFD_ICC_PRESENT;
+			return_value = IFD_ICC_PRESENT;
+			goto clean_up_and_return;
 		}
 	}
 	else
 	{
 		DEBUG_PERIODIC("Card absent");
 		// Clear ATR buffer
-		pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].nATRLength = 0;
-		*pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].
-			pcATRBuffer = '\0';
+		pgSlots[LunToReaderIndex(Lun)].nATRLength = 0;
+		*pgSlots[LunToReaderIndex(Lun)].pcATRBuffer = '\0';
 		// Card removed, clear the flags
-		pgSlots[LunToReaderIndex(Lun)][GCoreLunToSlotNb(Lun)].bPowerFlags =
+		pgSlots[LunToReaderIndex(Lun)].bPowerFlags =
 			POWERFLAGS_RAZ;
 
-		return IFD_ICC_NOT_PRESENT;
+		return_value = IFD_ICC_NOT_PRESENT;
 	}
+
+clean_up_and_return:
+	return return_value;
 } /* IFDHICCPresence */
 
+void IFDSetEmv(DWORD lun)
+{
+	pgSlots[LunToReaderIndex(lun)].bPowerFlags |= MASK_POWERFLAGS_EMV;
+} /* IFDSetEmv */
+
