@@ -7,7 +7,7 @@
 /            See http://www.linuxnet.com for more information.
 / License:   See file LICENSE.BSD
 /
-/ $Id: pcscdefines.h,v 1.3 2001/11/15 21:31:35 rousseau Exp $
+/ $Id: pcscdefines.h,v 1.5 2004/02/02 07:55:01 rousseau Exp $
 /
 ******************************************************************/
 
@@ -38,9 +38,9 @@ extern "C" {
     STATUS_DEVICE_PROTOCOL_ERROR = 0xFD
   } status_t;
 
-  #define MAX_RESPONSE_SIZE  264
-  #define MAX_ATR_SIZE       33
-  #define PCSCLITE_MAX_CHANNELS           16      /* Maximum channels     */
+#define MAX_RESPONSE_SIZE  264
+#define MAX_ATR_SIZE       33
+#define PCSCLITE_MAX_READERS           16      /* Maximum readers */
 #ifdef __cplusplus
 }
 #endif
